# Zpool

## Objetivo
Documentar instalação do zfsutils-linux para uso do binário zpool, utilizado para configuração de RAIDS e cacheamento


## Links
1. [ZFS](https://wiki.debian.org/ZFS)
2. [modprobe zfs](https://unix.stackexchange.com/questions/635413/debian-zfs-installation-error-module-zfs-not-found-in-directory-lib-modules-4)
3. [Setup a ZFS storage pool](https://ubuntu.com/tutorials/setup-zfs-storage-pool#1-overview)
4. [Configuring ZFS Cache for High Speed IO](https://linuxhint.com/configuring-zfs-cache/)


## Dependências
1. zfs-2.2.3-1  
2. zfs-dkms-2.2.3-1  


## Instalação
Para instalar, adicione aos backports `contrib` e `non-free-firmware` em `/etc/apt/sources.list`
```
codename=$(lsb_release -cs);echo "deb http://deb.debian.org/debian $codename-backports main contrib non-free"|sudo tee -a /etc/apt/sources.list && sudo apt update
```

Em seguida instale com os seguintes comandos:
```bash
  sudo apt update
  sudo apt install linux-headers-amd64 zfs-dkms
  sudo apt install -t stable-backports zfsutils-linux
```

Caso tenha problemas com modulo do `zfs` use `modprobe` ou reinstale.
```bash
# Modprobe
sudo modprobe zfs

# Reinstall
sudo apt purge --auto-remove zfsutils-linux 
sudo apt install zfs-dkms
sudo apt install zfsutils-linux
```


### Instalação Arch
```bash
# Install AUR packages
yay -S dkms zfs-dkms linux-headers

# Load the modules
modprobe zfs

# Init tanks after reboot
sudo systemctl enable zfs-import-cache.service
sudo systemctl enable zfs-mount.service
sudo systemctl enable zfs.target
```

Caso ocorra um problema de atualização com o kernel, como a falta de suporte a um kernel mais recente, utilize a versao de `long term suport (lts)` do kernel do arch.
```bash
# Install the kernel
sudo pacman -S linux-lts linux-lts-headers

# Update the grub
sudo grub-mkconfig -o /boot/grub/grub.cfg

sudo reboot
```


## Gerenciamento
Verificar status e historico de alterações das pools
```bash
zpool status
zpool history
```

Também é possível verificar os parâmetros de cada `pool`.
```bash
zfs set quota=2T tank/stage
zfs get compression tank
```


## RAID0
Para configurar discos em RAID0 (2 discos de gravação).
```bash
# Check WWN
lsblk -o NAME,TYPE,SIZE,WWN | grep disk
# Create tank
zpool create -f tank wwn-0x500.. wwn-0x500..
# Mount the tank
zfs create -o recordsize=16k -o compression=off -o atime=off -o xattr=sa -o logbias=latency -o redundant_metadata=most -o mountpoint=/home/stage tank/stage
```


## RAID10
Para configurar discos em RAID10 (4 discos, dois de gravação RAID0 e dois de espelhamento RAID1).  
Observe que na criação do tank é utilizado a opção de `mirror`
```bash
# Check WWN
lsblk -o NAME,TYPE,SIZE,WWN | grep disk
# Create tank
zpool create tank mirror wwn-0x500.. wwn-0x500.. mirror wwn-0x500.. wwn-0x500..
# Mount the tank
zfs create -o compression=lz4 -o atime=off -o xattr=sa -o logbias=latency -o redundant_metadata=most -o mountpoint=/home/stage tank/stage
```


## L2ARC
Adicione ao `tank` discos para cacheamento de leitura e escrita.
```bash
zpool add tank log wwn-0x500..
zpol add tank cache wwn-0x500..
```


## Removendo discos\Destruindo pool
Caso queira remover os discos de uma pool utilize `remove`.
Caso queira remover toda uma pool e liberar os discos utilize `destroy`.
```bash
# Remove disk
zpool remove tank wwn-0x500a5008abcdef8
# Remove pool
zpool destroy tank
```