# Kubernetes
Informações gerais sobre Kubernetes


## Links
1. [Digital Ocean Tutorials](https://www.digitalocean.com/community/tutorials)

## Recursos de um cluster
1. Namespace: Espaço de trabalho de uma aplicação
2. Pods: Container de uma aplicação
3. PV: Persistent Volumes
4. PVC: Persistent Volume Claims
5. Deployments: Gerenciadores de uma aplicação
6. StatefulSet: ...
7. Secrets: Dados de segurança
8. Services: Conexão
9. ConfigMap: Mapeamento de configuração
10. Ingress: API para gerenciamento de acesso externo


## Minikube
Minikube é um gerenciador de Kubernetes para testes que pode ser utilizado com apenas uma máquina.  
Para executá-lo é necessario um gerenciador de contêineres como docker.
```bash
# Check if minikube is not running
minikube delete
docker ps -a

# Start minikube
minikube start --cpus=4 --memory=8192

# Dashboard (monitoring)
minikube dashboard
```


## Ações básicas
Para gerenciar um cluster, é possivel utilizar comandos como `get`, `describe`, `log`, `exec`, `apply` e `delete`.
```bash
# List resources (-A for all namespaces)
kubectl get namespaces
kubectl get pv -n <namespace>
kubectl get pvc -n <namespace>
kubectl get pods -n <namespace>
kubectl get deployments -n <namespace>
kubectl get services -n <namespace>
kubectl get <resource> -n <namespace>
...

# Describe
kubectl describe pod <podname> -n <namespace>
kubectl describe deployment <deploymentname> -n <namespace>
kubectl describe <resource> -n <namespace>
...

# Enter in a `pod`
kubectl exec <podname> -it -n <namespace> -- bash
kubectl exec <podname> -it -n <namespace> -- <command>
...

# Apply a yaml file
kubectl apply -f setup_file.yaml
```


## Acessando o serviço de um `pod`
Quando uma aplicação está em funcionamento, é necessário configurar um `service` para que ele esteja visível as outras aplicações. Uma vez com o `service` criado é possível que outros pods possam acessar a aplicação.

Existem basicamente três tipos de `services`
- ClusterIP
- NodePort
- LoadBalancer

Também é possivel conectar a porta de um serviço ao localhost da máquina.
```bash
# Bind the pod port with the local host (useful for ssh tunneling)
kubectl port-foward <podname> <local_port>:<service_port>
kubectl port-foward service/<service_name> <local_port>:<service_port>
```



## Install cert-manager
Iniciar certificados para uso de criptografia.
```bash
# Add the Jetstack Helm repository
helm repo add jetstack https://charts.jetstack.io

# Update your local Helm chart repository cache
helm repo update

# Install the cert-manager Helm chart
helm install cert-manager jetstack/cert-manager \
  --namespace cert-manager \
  --create-namespace \
  --set crds.enabled=true
```



## Persistent Volume External Provisioner (Digital Ocean)
Sempre que um `persistent volume claim` é criado também é necessario que um `persistent volume` seja criado e conectado. Entretando, utilizando gerenciadores é possivel que apenas o `claim` seja criado e assim automaticamente o volume persistente seja gerado.



## Acessar serviço externo ao minikube
Caso seja necessário que um pod acesse um serviço externo ao minikube, alguns parâmetros de configuração devem ser alterados.

Busque o endereço ip do host que hospeda o minikube.
```bash
ifconfig
```

Dentro do arquivo de deployment, especifique a url com o ip e a porta.
```yaml
# Exemplo com elasticsearch
elasticsearch:
  host: "http://192.168.1.10:9200"  # Replace with your host's IP or DNS name
  port: 9200
```
