# Kubernetes Openmetadata Setup

Para configurar o openmetadata em kubernetes siga os seguintes passos.

Inicialmente crie todos os namespaces.
```bash
kubectl create namespace postgres
kubectl create namespace elasticsearch
kubectl create namespace openmetadata
kubectl create namespace airflow
```

## PostgreSQL
Suba uma instância de PostgreSQL conforme a subpasta `Kubernetes/postgres`. Com isso será criado uma instância com usuário `postgres` e senha `postgres`.

Em seguida carregue os dados relacionais do openmetadata.
```bash
kubectl exec -it postgres-0 -n postgres -- psql -U postgres < pg_opmntdata.sql
```

## Elasticsearch
Suba uma instância do Elasticsearch conforme subpasta `Kubernetes/elasticsearch`. Com isso será criado uma instância com usuário `elastic` e senha `elastic`.

Aguarde o serviço estar totalmente funcional. Em alguns é possivel que seja necessario deletar e criar de novo o statefulset (quase sempre na verdade).
Verifique a saúde com:
```bash
curl -X GET -u elastic:elastic "http://<elasticsearch-service-ip>:9200/"
```

Em seguida carregue os dados de índices executando o deployment de carregamento em `Kubernetes/openmetadata/load_data/execute-migrate-all-deployment.yaml`.  
Leva algum tempo até que todos os índices e documentos sejam criados.
```bash
kubectl apply -f execute-migrate-all-deployment.yaml
```

Aguarde até que o status do pod seja `completed` ou `crashLoopBackOff`.  
O pod precisa ser executado apenas uma vez, entretando como ele reinicia sempre que termina sua execução, ele precisa ser deletado na mão.  
Execute para terminar:
```bash
kubectl delete -f execute-migrate-all-deployment.yaml
```

## Openmetadata
Uma vez com os serviços do postgres e do elasticsearch funcionais, o serviço do openmetadata já pode ser subido:
```bash
kubectl apply -f Kubernetes/openmetadata/opmtdata_service.yaml
kubectl apply -f Kubernetes/openmetadata/opmtdata_deployment.yaml
```

Com isso a interface gráfica ja será acessível pelo IP do serviço e a porta 8585. Entretando, o openmetadata utiliza o airflow para se conectar a novos serviços.

## Airflow - Openmetadata
Para configurar o airflow do openmetadata utilize os arquivos em `Kubernetes/openmetadata/airflow/`.  

Uma vez que o airflow esteja funcional, copie os arquivos de `dags` em `Kubernetes/openmetadata/airflow/dagfiles/` para dentro do pod em `/opt/airflow/dags`.

Ao fim de tudo, o serviço do openmetadata deverá estar totalmente funcional. Para criar novas conexões se atente que para o openmetadata se conectar a um serviço ele deve estar visível ao pod.




