# Prometheus


## Objetivo
Criar um painel (dashboard) que apresente as métricas de vários servidores Postgresql, Clickhouse e que envie notificações de alerta para o discord ou telegram.


## Links
1. [Prometheus](https://schh.medium.com/monitoring-postgresql-databases-using-postgres-exporter-along-with-prometheus-and-grafana-1d68209ca687)
2. [Prometheus](https://www.ashnik.com/monitoring-postgresql-with-prometheus-and-grafana/)
3. [Postgres Dashboard](https://grafana.com/grafana/dashboards/9628-postgresql-database/)
4. [Clickhouse Dashboard](https://grafana.com/grafana/dashboards/14192-clickhouse/)
5. [Grafana alert template](https://faun.pub/overview-of-grafana-alerting-and-message-templating-for-slack-6bb740ec44af)


## Dependências
1. postgres_exporter, 0.17.0, Exportador de métricas do Postgres
2. promehteus, 2.48.1, Sistema de monitoramento e Banco de Dados Timeseries
3. grafana, 10.2.3, Dashboards


## Instalação
Para instalar o Prometheus é preciso baixar os binários e configurá-los.

**OBS:** Embora possa ter problemas com atualizações no futuro, binários de gerenciadores como APT estão longe da última versão suportada.
```bash
# Create prometheus user
useradd –-no-create-home –-shell /bin/false prometheus
mkdir /etc/prometheus
mkdir /var/lib/prometheus
chown prometheus:prometheus /etc/prometheus
chown prometheus:prometheus /var/lib/prometheus

# Download source
cd /opt
wget https://github.com/prometheus/prometheus/releases/download/v3.2.1/prometheus-3.2.1.linux-amd64.tar.gz
tar -xzf prometheus-3.2.1.linux-amd64.tar.gz
mv prometheus-3.2.1.linux-amd64/ prometheus
rm prometheus-3.2.1.linux-amd64.tar.gz
cd prometheus
```

Crie o arquivo `/opt/prometheus/prometheus.yml`
```yaml
# my global config
global:
  scrape_interval: 15s # Set the scrape interval to every 15 seconds. Default is every 1 minute.
  evaluation_interval: 15s # Evaluate rules every 15 seconds. The default is every 1 minute.
  # scrape_timeout is set to the global default (10s).

# Alertmanager configuration
alerting:
  alertmanagers:
    - static_configs:
        - targets:
          # - alertmanager:9093

# Load rules once and periodically evaluate them according to the global 'evaluation_interval'.
rule_files:
  # - "first_rules.yml"
  # - "second_rules.yml"

# A scrape configuration containing exactly one endpoint to scrape:
# Here it's Prometheus itself.
scrape_configs:
  # The job name is added as a label `job=<job_name>` to any timeseries scraped from this config.
  - job_name: "prometheus"

    # metrics_path defaults to '/metrics'
    # scheme defaults to 'http'.

    static_configs:
      - targets: ["localhost:9090"]

# === Add connections here

  - job_name: "postgres_exporter"
    static_configs:
      - targets: ["localhost:9187"]
```

Crie o arquivo `/etc/systemd/system/prometheus.service`

```service
[Unit]
Description=Prometheus
Wants=network-online.target
After=network-online.target

[Service]
User=prometheus
Group=prometheus
Type=simple
ExecStart=/opt/prometheus/prometheus \
  --config.file /opt/prometheus/prometheus.yml \
  --storage.tsdb.path /var/lib/prometheus/ \
  --web.console.templates=/etc/prometheus/consoles \
  --web.console.libraries=/etc/prometheus/console_libraries \
  --storage.tsdb.retention.time=1y

[Install]
WantedBy=multi-user.target
```

Inicie o serviço

```bash
systemctl daemon-reload
systemctl start prometheus
```


## Exportadores de Métricas
Para que o Prometheus possa armazenar dados cada serviço necessita de um exportador de métricas.

#### postgres_exporter
PostgreSQL não possui um exportador de métricas, logo é necessário configurar um mantido pelo próprio Prometheus.
```bash
# Download binaries
cd /opt
wget https://github.com/prometheus-community/postgres_exporter/releases/download/v0.17.0/postgres_exporter-0.17.0.linux-amd64.tar.gz
tar -xzf postgres_exporter-0.17.0.linux-amd64.tar.gz
mv postgres_exporter-0.17.0.linux-amd64 postgres_exporter
rm postgres_exporter-0.17.0.linux-amd64.tar.gz

# Change permissions
chown -R postgres:postgres postgres_exporter
cd postgres_exporter/
```

Crie o arquivo `/opt/postgres_exporter/postgres_exporter.env` para configurar o acesso ao banco (localhost no exemplo abaixo).

```env
DATA_SOURCE_NAME="postgresql://postgres:postgres@localhost:5432/?sslmode=disable"
```

Crie o arquivo `/etc/systemd/system/postgres_exporter.service` para o serviço.

```
[Unit]
Description=Prometheus exporter for Postgresql
Wants=network-online.target
After=network-online.target

[Service]
User=postgres
Group=postgres
WorkingDirectory=/opt/postgres_exporter
EnvironmentFile=/opt/postgres_exporter/postgres_exporter.env
ExecStart=/opt/postgres_exporter/postgres_exporter
Restart=always

[Install]
WantedBy=multi-user.target
```

Finalmente inicie o serviço:
```bash
systemctl daemon-reload
systemctl start postgres_exporter
```

**OBS:** Adicionar metricas postgres_exporter
Para adicionar mais metricas ao conjunto que é disponibilizado por padrão ao postgres_exporter, basta adicionar a flag da métrica que se deseja adicionar da lista em [Flags](https://github.com/prometheus-community/postgres_exporter?tab=readme-ov-file#flags) a variável `ExecStart` arquivo service do postgres_exporter.

Exemplo com postmaster:

```
[Unit]
Description=Prometheus exporter for Postgresql
Wants=network-online.target
After=network-online.target

[Service]
User=postgres
Group=postgres
WorkingDirectory=/opt/postgres_exporter
EnvironmentFile=/opt/postgres_exporter/postgres_exporter.env
ExecStart=/opt/postgres_exporter/postgres_exporter --collector.postmaster
Restart=always

[Install]
WantedBy=multi-user.target
```



#### Clickhouse
Clickhouse ja vem com um exportador de métricas e portanto é necessário apenas configura-lo.

Acesse o arquivo de configuração `config.xml` e descomente a tag prometheus com a porta especificada e reinicie o serviço.

```xml
    <prometheus>
        <endpoint>/metrics</endpoint>
        <port>9363</port>

        <metrics>true</metrics>
        <events>true</events>
        <asynchronous_metrics>true</asynchronous_metrics>
    </prometheus>
```

Em seguida adicione as `scrape_configs` do prometheus o `job_name` conforme o host e a porta.

```conf
scrape_configs:
  # The job name is added as a label `job=<job_name>` to any timeseries scraped from this config.
  - job_name: "prometheus"

    # metrics_path defaults to '/metrics'
    # scheme defaults to 'http'.

    static_configs:
      - targets: ["localhost:9090"]

  - job_name: "clickhouse"
    static_configs:
      - targets: ["localhost:9363"]

```

Finalmente reinicie o serviço do prometheus.

